<?php

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Class GelatoStatusChecker
 */
class GelatoStatusChecker
{
    public const STATUS_OK = 'OK';
    public const STATUS_FAIL = 'FAIL';
    public const STATUS_WARNING = 'WARNING';
    public const STATUS_SKIPPED = 'SKIPPED';

    public function getTestList() {
        return [
            [
                'name'        => __('WordPress Permalinks', 'gelato-integration-for-woocommerce'),
                'description' => __('Ensure that your permalinks are NOT set to "plain" in Settings > Permalinks. Any other setting than "plain" is ok.', 'gelato-integration-for-woocommerce'),
                'method'      => 'test_permalinks',
            ],
            [
                'name'        => __('WooCommerce Webhooks', 'gelato-integration-for-woocommerce'),
                'description' => __('Gelato requires webhooks in WooCommerce to be set up to capture your orders, product updates etc in close to real-time.', 'gelato-integration-for-woocommerce'),
                'method'      => 'test_gelato_webhooks',
            ],
            [
                'name'        => __('WooCommerce Gelato API keys are set and valid.', 'gelato-integration-for-woocommerce'),
                'description' => __('For the integration to work Gelato needs access to your WooCommerce API - otherwise we can\'t keep your store updated with products.', 'gelato-integration-for-woocommerce'),
                'method'      => 'test_wc_api_access_by_gelato',
            ],
            [
                'name'        => __('Check connection with Gelato.', 'gelato-integration-for-woocommerce'),
                'description' => __('We check that connection between Gelato and WooCommerce accessible via API.', 'gelato-integration-for-woocommerce'),
                'method'      => 'test_connection_from_gelato_to_wc',
            ],
            [
                'name'        => __('Write permissions', 'gelato-integration-for-woocommerce'),
                'description' => __('To be able to push mockups to your store the upload directory needs to be writable. If you need help with this contact your hosting provider.', 'gelato-integration-for-woocommerce'),
                'method'      => 'test_uploads_write',
            ],
            [
                'name'        => __('PHP memory limit', 'gelato-integration-for-woocommerce'),
                'description' => __('Set PHP allocated memory limit to at least 128mb. Contact your hosting provider if you need help with this.', 'gelato-integration-for-woocommerce'),
                'method'      => 'test_php_memory_limit',
            ],
            [
                'name'        => __('PHP script time limit', 'gelato-integration-for-woocommerce'),
                'description' => __('Set PHP script execution time limit to at least 30 seconds. This is required to add products with many variants. Contact your hosting provider if you need help with this.', 'gelato-integration-for-woocommerce'),
                'method'      => 'test_php_max_execution_time',
            ],
            [
                'name'        => __('Check SSL', 'gelato-integration-for-woocommerce'),
                'description' => __('We are checking is everything is ok with SSL.', 'gelato-integration-for-woocommerce'),
                'method'      => 'test_ssl',
            ],
            [
                'name'        => __('Check site redirection.', 'gelato-integration-for-woocommerce'),
                'description' => __('There are might be some issues if your site is configured with redirection to another URL. Usually this happens with incorrect http to https redirects or yourdomain.com to www.yourdomain.com. Please check your settings.', 'gelato-integration-for-woocommerce'),
                'method'      => 'test_redirection',
            ],
        ];
    }

    public function getTestResults(): array
    {
        $results = [];
        foreach ($this->getTestList() as $item) {
            $result = $item;
            $result['status'] = self::STATUS_SKIPPED;

            if (method_exists($this, $item['method'])) {
                $result['status'] = $this->{$item['method']}();
            }
            $results[] = $result;
        }

        return $results;
    }

    private function test_permalinks()
    {
        $permalinks = get_option('permalink_structure', false);

        if ($permalinks && strlen($permalinks) > 0) {
            return self::STATUS_OK;
        }

        return self::STATUS_FAIL;
    }

    private function test_gelato_webhooks()
    {
        global $wpdb;

        $count = $wpdb->get_var($wpdb->prepare(
            "SELECT COUNT(*) as webhook_count FROM {$wpdb->prefix}wc_webhooks WHERE name LIKE '%%%s%'",
            "Gelato |"
        ));

        if ($count == 6) {
            return self::STATUS_OK;
        }

        if ($count > 0) {
            return self::STATUS_WARNING;
        }

        return self::STATUS_FAIL;
    }

    private function test_wc_api_access_by_gelato()
    {
        global $wpdb;

        $key = $wpdb->get_row($wpdb->prepare(
            "SELECT * FROM {$wpdb->prefix}woocommerce_api_keys WHERE description LIKE '%%%s%' ORDER BY last_access LIMIT 1",
            $wpdb->esc_like("Gelato")
        ));

        if (!empty($key) && $key->permissions == 'read_write') {
            return self::STATUS_OK;
        }

        return self::STATUS_FAIL;
    }

    private function test_connection_from_gelato_to_wc()
    {
        $url = GelatoApiClientFactory::GELATO_HOST . GelatoApiClientFactory::GELATO_API_STATUS_URL;
        $response = wp_remote_get(
            $url . "?host=" . get_option('siteurl'),
            [
                'host' => get_option('siteurl')
            ]
        );
        $bodyResponse = wp_remote_retrieve_body($response);
        $responseCode = wp_remote_retrieve_response_code($response);

        if ($responseCode !== 200) {
            return self::STATUS_FAIL;
        }

        $response = json_decode($bodyResponse, true);

        if ($response['status'] == 'OK') {
            return self::STATUS_OK;
        }

        return self::STATUS_FAIL;
    }

    public function test_php_max_execution_time()
    {
        $maxExecTime = ini_get('max_execution_time');

        if (!$maxExecTime || $maxExecTime >= 30) {
            return self::STATUS_OK;
        }

        return self::STATUS_FAIL;
    }

    private function test_php_memory_limit()
    {
        $memoryLimit = ini_get('memory_limit');

        if (preg_match('/^(\d+)(.)$/', $memoryLimit, $matches)) {
            if ($matches[2] == 'M') {
                $memoryLimit = $matches[1] * 1024 * 1024;
            } else {
                if ($matches[2] == 'K') {
                    $memoryLimit = $matches[1] * 1024;
                }
            }
        }

        $ok = ($memoryLimit >= 128 * 1024 * 1024);

        if ($ok) {
            return self::STATUS_OK;
        }

        return self::STATUS_FAIL;
    }

    private function test_ssl()
    {
        if (is_ssl()) {
            return self::STATUS_OK;
        }

        return self::STATUS_WARNING;
    }

    private function test_uploads_write()
    {
        $uploadDir = wp_upload_dir();
        if (is_writable($uploadDir['basedir'])) {
            return self::STATUS_OK;
        }

        return self::STATUS_OK;
    }

    private function test_redirection()
    {
        $siteUrl = get_option('siteurl');
        $requestBasic = wp_remote_head( $siteUrl . '/wp-json/', ['redirection' => 0]);
        $requestSlashed = wp_remote_head( trailingslashit($siteUrl), ['redirection' => 0]);

        if (is_wp_error($requestBasic) || is_wp_error($requestSlashed)) {
            return self::STATUS_FAIL;
        }

        /** @var WP_HTTP_Requests_Response $response */
        $responseBasic = $requestBasic['http_response'];
        /** @var WP_HTTP_Requests_Response $responseSlashed */
        $responseSlashed = $requestSlashed['http_response'];

        if ($responseBasic->get_status() == 200 || $responseSlashed->get_status() == 200) {
            return self::STATUS_OK;
        }

        if (in_array($responseBasic->get_status(), array(301, 302, 303, 307))) {
            return self::STATUS_FAIL;
        }

        return self::STATUS_WARNING;
    }
}
